/*
 * Copyright 2016 Advanced Micro Devices, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#ifndef __ADL_H__
#define __ADL_H__

#define ADL_MAX_PATH				256

/// Defines ADL_TRUE
#define ADL_TRUE				1
/// Defines ADL_FALSE
#define ADL_FALSE				0

/// \defgroup define_adl_results Result Codes
/// This group of definitions are the various results returned by all ADL functions \n
// @{
/// All OK, but need to wait
#define ADL_OK_WAIT				4
/// All OK, but need restart
#define ADL_OK_RESTART				3
/// All OK but need mode change
#define ADL_OK_MODE_CHANGE			2
/// All OK, but with warning
#define ADL_OK_WARNING				1
/// ADL function completed successfully
#define ADL_OK					0
/// Generic Error. Most likely one or more of the Escape calls to the driver failed!
#define ADL_ERR				-1
/// ADL not initialized
#define ADL_ERR_NOT_INIT			-2
/// One of the parameter passed is invalid
#define ADL_ERR_INVALID_PARAM			-3
/// One of the parameter size is invalid
#define ADL_ERR_INVALID_PARAM_SIZE		-4
/// Invalid ADL index passed
#define ADL_ERR_INVALID_ADL_IDX		-5
/// Invalid controller index passed
#define ADL_ERR_INVALID_CONTROLLER_IDX		-6
/// Invalid display index passed
#define ADL_ERR_INVALID_DIPLAY_IDX		-7
/// Function  not supported by the driver
#define ADL_ERR_NOT_SUPPORTED			-8
/// Null Pointer error
#define ADL_ERR_NULL_POINTER			-9
/// Call can't be made due to disabled adapter
#define ADL_ERR_DISABLED_ADAPTER		-10
/// Invalid Callback
#define ADL_ERR_INVALID_CALLBACK        	-11
/// Display Resource conflict
#define ADL_ERR_RESOURCE_CONFLICT		-12
//Failed to update some of the values. Can be returned by set request that include multiple values if not all values were successfully committed.
#define ADL_ERR_SET_INCOMPLETE 		-20
/// There's no Linux XDisplay in Linux Console environment
#define ADL_ERR_NO_XDISPLAY			-21

typedef void* (*ADL_MAIN_MALLOC_CALLBACK)(int);

/**
 * \brief Structure containing information about the graphics adapter.
 *
 *  This structure is used to store various information about the graphics adapter.
 *  This information can be returned to the user.
 *  Alternatively, it can be used to access various driver calls to set
 *  or fetch various settings upon the user's request.
 *  \nosubgrouping
*/
typedef struct AdapterInfo
{
	/// \ALL_STRUCT_MEM

	/// Size of the structure.
	int iSize;
	/// The ADL index handle. One GPU may be associated with one or two index handles
	int iAdapterIndex;
	/// The unique device ID associated with this adapter.
	char strUDID[ADL_MAX_PATH];
	/// The BUS number associated with this adapter.
	int iBusNumber;
	/// The driver number associated with this adapter.
	int iDeviceNumber;
	/// The function number.
	int iFunctionNumber;
	/// The vendor ID associated with this adapter.
	int iVendorID;
	/// Adapter name.
	char strAdapterName[ADL_MAX_PATH];
	/// Display name. For example, "\\Display0" for Windows or ":0:0" for Linux.
	char strDisplayName[ADL_MAX_PATH];
	/// Present or not; 1 if present and 0 if not present.It the logical adapter is present, the display name such as \\.\Display1 can be found from OS
	int iPresent;
	// @}

#if defined (_WIN32) || defined (_WIN64)
	/// \WIN_STRUCT_MEM

	/// Exist or not; 1 is exist and 0 is not present.
	int iExist;
	/// Driver registry path.
	char strDriverPath[ADL_MAX_PATH];
	/// Driver registry path Ext for.
	char strDriverPathExt[ADL_MAX_PATH];
	/// PNP string from Windows.
	char strPNPString[ADL_MAX_PATH];
	/// It is generated from EnumDisplayDevices.
	int iOSDisplayIndex;
	// @}
#endif /* (_WIN32) || (_WIN64) */

#if defined (LINUX)
	/// \LNX_STRUCT_MEM

	/// Internal X screen number from GPUMapInfo (DEPRICATED use XScreenInfo)
	int iXScreenNum;
	/// Internal driver index from GPUMapInfo
	int iDrvIndex;
	/// \deprecated Internal x config file screen identifier name. Use XScreenInfo instead.
	char strXScreenConfigName[ADL_MAX_PATH];

	// @}
#endif /* (LINUX) */
	} AdapterInfo, *LPAdapterInfo;

/**
 * \brief Structure containing information about the ASIC memory.
 *
 *  This structure is used to store various information about the ASIC memory.  This
 *  information can be returned to the user.
 *  \nosubgrouping
*/
typedef struct ADLMemoryInfo
{
	/// Memory size in bytes.
	long long iMemorySize;
	/// Memory type in string.
	char strMemoryType[ADL_MAX_PATH];
	/// Memory bandwidth in Mbytes/s.
	long long iMemoryBandwidth;
} ADLMemoryInfo, *LPADLMemoryInfo;

/**
 * \brief Function to initialize the ADL interface. This function should be called first.
 *
 * This function initializes the ADL library. ADL initialized using ADL_Main_Control_Create
 * will not enforce serialization of ADL API executions by multiple threads.
 * Multiple threads will be allowed to enter to ADL at the same time.
 * Note that ADL library is not guaranteed to be thread-safe. Client that calls
 * ADL_Main_Control_Create have to provide its own mechanism for ADL calls serialization.
 * It also initializes global pointers and, at the same time, calls the ADL_Main_Control_Refresh function.
 * \platform
 * \ALL
 * \param[in] callback: The memory allocation function for memory buffer allocation.
 *                              This must be provided by the user. ADL internally uses this callback to
 *                              allocate memory for the output parameters returned to the user and
 *                              user is responsible to free the memory once used for these parameters.
 *                              ADL internal takes care of allocating and de allocating the memory for its local variables.
 * \param[in] iEnumConnectedAdapters: Specify a value of 0 to retrieve adapter information
 *                              for all adapters that have ever been present in the system. Specify a value of 1 to
 *                              retrieve adapter information only for adapters that are physically present and enabled in the system.
 * \return If the function succeeds, the return value is \ref ADL_OK. Otherwise the return value is an ADL error code. \ref define_adl_results
*/
int ADL_Main_Control_Create(ADL_MAIN_MALLOC_CALLBACK callback, int iEnumConnectedAdapters);

/**
 * \brief Function to destroy ADL global pointers. This function should be called last.
 *
 * All ADL global buffers and resources are released after this function is called.
 * \platform
 * \ALL
 * \return If the function succeeds, the return value is \ref ADL_OK.
 *           Otherwise the return value is an ADL error code. \ref define_adl_results
*/
int ADL_Main_Control_Destroy(void);

/**
 * \brief Function to retrieve the number of OS-known adapters.
 *
 * This function retrieves the number of graphics adapters recognized by the OS (OS-known adapters).
 * OS-known adapters can include adapters that are physically present in the system (logical adapters)
 * as well as ones that no longer present in the system but are still recognized by the OS.
 * \platform
 * \ALL
 * \param[out] lpNumAdapters The pointer to the number of OS-known adapters.
 * \return If the function succeeds, the return value is \ref ADL_OK.
 *            Otherwise the return value is an ADL error code. \ref define_adl_results
*/
int ADL_Adapter_NumberOfAdapters_Get(int* lpNumAdapters);

/**
 * \brief Function to determine if the adapter is active or not.
 *
 * The function is used to check if the adapter associated with iAdapterIndex is active. Logic is different for Windows and Linux!
 * \platform
 * \ALL
 * \param[in]  iAdapterIndex The ADL index handle of the desired adapter.
 * \param[out] lpStatus The pointer to the retrieved status. \ref ADL_TRUE : Active; \ref ADL_FALSE : Disabled.
 * \return If the function succeeds, the return value is \ref ADL_OK.
 *            Otherwise the return value is an ADL error code. \ref define_adl_results
*/
int ADL_Adapter_Active_Get(int iAdapterIndex, int* lpStatus);

/**
 * \brief Retrieves all OS-known adapter information.
 *
 * This function retrieves the adapter information of all OS-known adapters in the system.
 * OS-known adapters can include adapters that are physically present in the system (logical adapters)
 * as well as ones that are no longer present in the system but are still recognized by the OS.
 * \platform
 * \ALL
 * \param[in]  iInputSize The size of the lpInfo buffer.
 * \param[out] lpInfo The pointer to the buffer containing the retrieved adapter information.
 * \return If the function succeeds, the return value is \ref ADL_OK. Otherwise the return value is an ADL error code. \ref define_adl_results
 * \remarks This API take a fixed-size output array. For dynamic-size output, use \ref ADL_Adapter_AdapterInfoX2_Get function.
*/
int ADL_Adapter_AdapterInfo_Get(LPAdapterInfo lpInfo, int iInputSize);

/**
 * \brief Function to retrieve memory information from the adapter.
 *
 * This function retrieves the memory information for a specified graphics adapter.
 * \platform
 * \LNX
 * \param[in]  iAdapterIndex The ADL index handle of the desired adapter.
 * \param[out] lpMemoryInfo The pointer to the structure storing the retrieved memory information.
 * \return If the function succeeds, the return value is \ref ADL_OK. Otherwise the return value is an ADL error code. \ref define_adl_results
*/
int ADL_Adapter_MemoryInfo_Get(int iAdapterIndex, ADLMemoryInfo *lpMemoryInfo);

#endif
